import { FileView } from "./FileView.tsx";
import { useState } from "react";

import {
  MdOutlineDelete,
  MdExpandLess,
  MdExpandMore,
  MdEdit,
  MdOutlineSave,
  MdOutlineContentCopy,
  MdOutlineInsertDriveFile,
  MdOutlineCancel,
} from "react-icons/md";

import RuleView from "./RuleView.tsx";
import RuleInput from "./RuleInput.tsx";
import React from "react";
import { RightSide, Rule } from "../App";
const GrammarView = ({
  grammar,
  deleteGr,
  editGr,
  addRule,
  duplicateGr,
  deleteRule,
  changeRule,
  generateFile,
  changeRoot,
  getFile,
  fileToGrammar,
  deleteFile,
  deleteFilesFromGrammar,
}) => {
  const emptyRule: Rule = {
    id: null,
    leftSymbol: null,
    root: null,
    rightSide: [],
  };
  const [more, setMore] = useState(false);
  const [ruleAdd, setRuleAdd] = useState(false);
  const [edit, setEdit] = useState(false);
  const [uri, setUri] = useState("");
  const [newRule, setNewRule] = useState(emptyRule);
  const [file, setFile] = useState(false);

  function handleSubmit(e) {
    e.preventDefault();
    if (uri === null || uri.length === 0) {
      return;
    }
    setEdit(false);
    var gr = { uri: uri, id: grammar.id };
    editGr(gr);
  }

  function handleNewRule(e) {
    e.preventDefault();
    if (
      newRule.rightSide === null ||
      newRule.leftSymbol === null ||
      newRule.rightSide[0].part.length === 0 ||
      newRule.leftSymbol.length === 0
    ) {
      return;
    }
    setRuleAdd(false);
    setNewRule(emptyRule);
    addRule(newRule, grammar.id);
  }

  function changeRuleAddGrammarId(ruleId: number, rule: Rule) {
    changeRule(ruleId, rule, grammar.id);
  }

  function switchEditmode() {
    if (edit) {
      setRuleAdd(false);
    } else {
      setMore(!edit);
    }

    setEdit(!edit);
  }
  const getStringRight = (rightSide: RightSide[]) => {
    let result = "";
    let i = 0;
    rightSide.forEach((element) => {
      result += element.part;
      if (i + 1 < rightSide.length) {
        result += " | ";
      }
      i++;
    });
    return result;
  };

  function deleteRuleAddGrammarId(ruleID: number, root: boolean) {
    deleteRule(ruleID, root, grammar.id);
  }

  function changeRootAddGrammarID(ruleId: number, rule: Rule) {
    changeRoot(ruleId, grammar.id, rule);
  }
  return (
    <>
      <div className="grammar">
        <article>
          <h3>
            {grammar.uri} {edit && <MdOutlineSave onClick={handleSubmit} />}{" "}
          </h3>
          {edit && (
            <form onSubmit={(e) => handleSubmit(e)}>
              <input
                className="transparent"
                onChange={(e) => setUri(e.target.value)}
                type="text"
                placeholder="Change uri"
              />
            </form>
          )}
          <p className="rootRuleSmall">
            {grammar.rootRule !== undefined &&
              grammar.rootRule.leftSymbol +
                " = " +
                getStringRight(grammar.rootRule.rightSide)}
          </p>
          <p>
            <MdOutlineInsertDriveFile onClick={() => setFile(!file)} />{" "}
            {more ? (
              <MdExpandLess onClick={() => setMore(!more)} />
            ) : (
              <MdExpandMore onClick={() => setMore(!more)} />
            )}
            <MdEdit onClick={switchEditmode} />{" "}
            <MdOutlineContentCopy onClick={() => duplicateGr(grammar.id)} />{" "}
            <MdOutlineDelete onClick={() => deleteGr(grammar.id)} />
          </p>
          {more &&
            grammar.rules.map((rule) => (
              <RuleView
                key={rule.id}
                rule={rule}
                deleteRule={deleteRuleAddGrammarId}
                changeRule={changeRuleAddGrammarId}
                masterRoot={grammar.rootRule !== null}
                changeRoot={changeRootAddGrammarID}
              />
            ))}
          {edit && (
            <>
              {!ruleAdd ? (
                <>
                  <button onClick={() => setRuleAdd(true)}>ADD RULE</button>{" "}
                </>
              ) : (
                <form onSubmit={handleNewRule}>
                  <RuleInput
                    rule={newRule}
                    setRule={setNewRule}
                    haveRoot={grammar.rootRule !== null}
                  />{" "}
                  <MdOutlineSave onClick={handleNewRule} />
                  <MdOutlineCancel onClick={() => setRuleAdd(false)} />
                </form>
              )}
            </>
          )}
          {file && (
            <FileView
              generateFile={generateFile}
              grammar={grammar}
              getFile={getFile}
              fileToGrammar={fileToGrammar}
              deleteFile={deleteFile}
              deleteFilesFromGrammar={deleteFilesFromGrammar}
            />
          )}
        </article>
      </div>{" "}
    </>
  );
};

export default GrammarView;
