package com.jakubu9333.grammarManager.api;

import com.fasterxml.jackson.annotation.JsonView;
import com.jakubu9333.grammarManager.model.FileHolder;
import com.jakubu9333.grammarManager.model.Grammar;
import com.jakubu9333.grammarManager.model.View;
import com.jakubu9333.grammarManager.service.FileService;
import org.javatuples.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * @author Jakub Uhlarik
 */

@RestController
@CrossOrigin(origins = "http://localhost:3000")
@RequestMapping(path = "api/files")
public class FilesControler {
    private final FileService fileService;

    @Autowired
    public FilesControler(FileService fileService) {
        this.fileService = fileService;
    }

    @DeleteMapping("/{id}")
    public ResponseEntity<?> deleteFile(@PathVariable Long id) {
        fileService.deleteFileById(id);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @DeleteMapping("/grammar/{id}")
    public ResponseEntity<?> deleteFileFromGrammar(@PathVariable Long id) {
        fileService.deleteFilesFromGrammar(id);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }
    @JsonView(View.All.class)
    @GetMapping("/grammar/{id}")
    public ResponseEntity<List<FileHolder>> fileHoldersFromGrammar(@PathVariable Long id){
        return new ResponseEntity<>(fileService.getFilesFromGrammar(id),HttpStatus.OK);
    }

    @GetMapping("/{id}")
    public ResponseEntity<InputStreamResource> getFile(@PathVariable Long id) throws IOException {
        MediaType typeOfFile;
        FileHolder fileHolder = fileService.getFileHolderById(id);
        if (fileHolder.isXml()) {
            typeOfFile = MediaType.APPLICATION_XML;
        } else {
            typeOfFile = MediaType.TEXT_PLAIN;
        }
        InputStream is = new FileInputStream("./grammars/" + fileHolder.getFilePath());
        return ResponseEntity.ok().contentType(typeOfFile).body(new InputStreamResource(is));
    }

    @GetMapping("/{id}/download")
    public ResponseEntity<InputStreamResource> getFileDownload(@PathVariable Long id, HttpServletResponse response) throws IOException {
        MediaType typeOfFile;
        FileHolder fileHolder = fileService.getFileHolderById(id);
        if (fileHolder.isXml()) {
            typeOfFile = MediaType.APPLICATION_XML;
        } else {
            typeOfFile = MediaType.TEXT_PLAIN;
        }
        InputStream is = new FileInputStream("./grammars/" + fileHolder.getFilePath());
        response.setHeader("Content-Disposition", "attachment; filename=" + fileHolder.getFilePath());
        return ResponseEntity.ok().contentType(typeOfFile).body(new InputStreamResource(is));
    }


    @GetMapping("{grammarId}/new/{fileFormat}")
    public ResponseEntity<InputStreamResource> fileFromGrammar(@PathVariable String fileFormat, @PathVariable Long grammarId) throws IOException {
        MediaType typeOfFile;
        boolean xml;
        switch (fileFormat) {
            case "jsgf" -> {
                typeOfFile = MediaType.TEXT_PLAIN;
                xml = false;
            }
            case "xml" -> {
                typeOfFile = MediaType.APPLICATION_XML;
                xml = true;
            }
            default -> throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "invalid file format");
        }
        Pair<FileHolder, Boolean> result = fileService.newFileFromGrammar(grammarId, xml);
        InputStream is = new FileInputStream("./grammars/" + result.getValue0().getFilePath());
        return ResponseEntity.status(result.getValue1() ? HttpStatus.CREATED : HttpStatus.OK).contentType(typeOfFile).body(new InputStreamResource(is));
    }

    @GetMapping("{grammarId}/new/{fileFormat}/download")
    public ResponseEntity<InputStreamResource> fileFromGrammarDownload(@PathVariable String fileFormat, @PathVariable Long grammarId, HttpServletResponse response) throws IOException {
        MediaType typeOfFile;
        boolean xml;
        switch (fileFormat) {
            case "jsgf" -> {
                typeOfFile = MediaType.TEXT_PLAIN;
                xml = false;
            }
            case "xml" -> {
                typeOfFile = MediaType.APPLICATION_XML;
                xml = true;
            }
            default -> throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "invalid file format");
        }
        FileHolder fileHolder = fileService.getGeneratedFile(grammarId, xml);
        InputStream is = new FileInputStream("./grammars/" + fileHolder.getFilePath());
        response.setHeader("Content-Disposition", "attachment; filename=" + fileHolder.getFilePath());
        return ResponseEntity.ok().contentType(typeOfFile).body(new InputStreamResource(is));
    }

    @JsonView(View.RuleWithoutGrammar.class)
    @PostMapping("/{fileFormat}")
    public ResponseEntity<Grammar> grammarFromFile(@RequestBody String file, @PathVariable String fileFormat, @RequestParam(defaultValue = "true") boolean save) throws IOException {
        boolean xml;
        switch (fileFormat) {
            case "jsgf" -> xml = false;

            case "xml" -> xml = true;

            default -> throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "invalid file format");
        }
        Grammar grammar = fileService.addGrammarFromFile(file, xml, null, false, save).getValue0();
        return new ResponseEntity<>(grammar, HttpStatus.CREATED);
    }

    @JsonView(View.RuleWithoutGrammar.class)
    @PostMapping("{id}/{fileFormat}")
    public ResponseEntity<Grammar> grammarFromFile(@RequestBody String file, @PathVariable String fileFormat, @PathVariable Long id, @RequestParam(defaultValue = "false") boolean newGrammar, @RequestParam(defaultValue = "true") boolean save) throws IOException {
        boolean xml;
        switch (fileFormat) {
            case "jsgf" -> xml = false;

            case "xml" -> xml = true;

            default -> throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "invalid file format");
        }
        Pair<Grammar,Boolean> result = fileService.addGrammarFromFile(file, xml, id, newGrammar, save);
        if (result == null) {
            return new ResponseEntity<>(HttpStatus.NO_CONTENT);
        }
        return new ResponseEntity<>(result.getValue0(), result.getValue1() ? HttpStatus.CREATED: HttpStatus.OK );
    }
}
