package com.jakubu9333.grammarManager.model;

import com.fasterxml.jackson.annotation.JsonView;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.SequenceGenerator;
import java.io.File;
import java.time.LocalDateTime;

/**
 * @author Jakub Uhlarik
 */
@Entity
@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
public class FileHolder {

    @Id
    @JsonView(View.All.class)
    @SequenceGenerator(name = "file_sequence", allocationSize = 1, sequenceName = "file_sequence")
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "file_sequence")
    private Long id;


    @JsonView(View.All.class)
    private String filePath;

    @JsonView(View.All.class)
    private boolean xml;

    @JsonView(View.All.class)
    private boolean generated;

    @JsonView(View.All.class)
    private long size;

    @ManyToOne
    private Grammar grammar;

    private LocalDateTime createTime;

    public FileHolder(String file, boolean xml, boolean generated, Grammar grammar) {
        this.filePath = file;
        this.xml = xml;
        this.generated = generated;
        this.grammar = grammar;
    }

    @PrePersist
    public void prePersist() {
        this.createTime = LocalDateTime.now();
    }

    @PreRemove
    public void preRemove() {
        new File("./grammars/" + this.filePath).delete();
    }
}
