package com.jakubu9333.grammarManager;

import com.jakubu9333.grammarManager.model.FileHolder;
import com.jakubu9333.grammarManager.model.Grammar;
import com.jakubu9333.grammarManager.model.RightSidePart;
import com.jakubu9333.grammarManager.model.Rule;
import com.jakubu9333.grammarManager.service.FileService;
import com.jakubu9333.grammarManager.service.GrammarService;

import com.jakubu9333.grammarManager.service.RuleService;
import org.javatuples.Pair;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.web.server.ResponseStatusException;

import javax.transaction.Transactional;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * @author Jakub Uhlarik
 */
@SpringBootTest
public class FileGrammarServiceTest {

    @Autowired
    public GrammarService grammarService;

    @Autowired
    public FileService fileService;
    @Autowired
    public RuleService ruleService;

    @Test
    public void jsgfReadTest() throws IOException {
        grammarService.deleteAll();
        boolean xml = false;
        Grammar emptyGrammar = fileService.addGrammarFromFile("", xml, null, false, false).getValue0();
        assert emptyGrammar.getUri().equals("null");
        assert emptyGrammar.getRules().size() == 0;
        assert emptyGrammar.getRootRule() == null;
        String file1 = """
                #JSGF;
                grammar first;
                rule1 = smile;
                //root;
                rule2 = cry|sis;
                                
                               
                          
                """;
        Grammar firstGrammar = fileService.addGrammarFromFile(file1, xml, null, false, false).getValue0();
        assert firstGrammar.getUri().equals("first");
        Rule rule2 = firstGrammar.getRootRule();
        assert rule2.getLeftSymbol().equals("<rule2>");
        assert firstGrammar.getRules().size() == 2;
        assert rule2.getRoot();
        assert rule2.getRightSide().equals(RuleServiceTest.getFromString("cry|sis"));
        Rule rule1 = firstGrammar.getRules().stream().filter(p -> p.getLeftSymbol().equals("<rule1>")).toList().get(0);
        assert rule1.getLeftSymbol().equals("<rule1>");
        assert rule1.getRightSide().equals(RuleServiceTest.getFromString("smile"));
        String file2 = """
                                
                grammar dms;
                ddsd=sfdsfd;
                grammar xfs;
                                
                """;
        boolean error = false;
        try {
            fileService.addGrammarFromFile(file2, xml, null, false, false);
        } catch (ResponseStatusException e) {
            assert e.getReason().matches("duplicate grammar name declaration; line=[0-9]+");
            error = true;
        }
        assert error;
        String file3 = """
                                
                grammar dms;
                //root;
                ddsd=sfdsfd;
                 //root;
                ddssfsd=sfdsfd;
                                
                """;
        error = false;
        try {
            fileService.addGrammarFromFile(file3, xml, null, false, false);
        } catch (ResponseStatusException e) {
            error = true;
        }
        assert error;
    }

    @Test
    public void xmlReadTest() throws IOException {
        grammarService.deleteAll();
        boolean xml = true;
        boolean er = false;
        try {
            fileService.addGrammarFromFile("", xml, null, false, false);
        } catch (ResponseStatusException e) {
            er = true;
        }
        assert er;

        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        Grammar firstGrammar = fileService.addGrammarFromFile(file1, xml, null, false, false).getValue0();
        assert firstGrammar.getUri().equals("first");
        for (Rule rule : firstGrammar.getRules()) {
            assert predicate1(rule);
        }


        String fileRoot = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                           root="rule1"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        Grammar rootGr = fileService.addGrammarFromFile(fileRoot, xml, null, false, false).getValue0();
        assert rootGr.getRootRule() != null;

    }

    @Test
    public void testErrorMessages() {


        String file = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="second"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>"123"
                            </item>
                            <item>"smile "<ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                     
                                <rule id="rule2">
                                    <item>"cry"</item>
                                </rule>
                            </grammar>
                          
                """;
        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            </rule>
                            </grammar>
                """;
        String message = "parsing error";
        String message1 = "parsing error";
        List<String> files = new ArrayList<>();
        List<String> messages = new ArrayList<>();
        files.add(file);
        files.add(file1);
        messages.add(message);
        messages.add(message1);
        String file2 = """
                            <rule id="2"> </rule>
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              </grammar>
                """;
        files.add(file2);
        messages.add("parsing error");

        String file3 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"><item> "ddd" </item>
                              </rule>
                              <one-of>
                              </one-of>
                              </grammar>
                """;
        files.add(file3);
        messages.add("parsing error");
        String file4 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"bubub"
                              </one-of>
                              </rule>
                             </item>
                              </grammar>
                """;
        files.add(file4);
        messages.add("parsing error");

        String file5 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"cry"
                             </item>
                              </rule>
                              </one-of>
                             
                              </grammar>
                """;
        files.add(file5);
        messages.add("parsing error");
        String file6 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"><item>"smile"</item>
                              </one-of>
                              </rule>
                             
                              </grammar>
                """;
        files.add(file6);
        messages.add("parsing error");
        String file7 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"xd"</item>
                              </one-of>
                              </rule>
                            <item>"outside"</item>
                              </grammar>
                """;
        messages.add("parsing error");
        files.add(file7);
        String file8 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"ff</item>
                              </one-of>
                              </rule>
                              <rule id="5>  <one-of><item>"ls</item>
                              </one-of>
                    
                              </grammar>
                """;
        messages.add("parsing error");
        files.add(file8);


        for (int i = 0; i < files.size(); i++) {

            testOneMessage(files.get(i), messages.get(i));
        }

    }

    private void testOneMessage(String file, String message) {

        boolean error = false;
        try {
            fileService.addGrammarFromFile(file, true, null, false, false);
        } catch (ResponseStatusException e) {
            assert e.getReason().equals(message);
            error = true;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        assert error;
    }

    private boolean predicate1(Rule rule) {
        boolean result;
        switch (rule.getLeftSymbol()) {
            case "<rule1>" ->
                    result = rule.getRightSide().equals(RuleServiceTest.getFromString("123|smile <rule2>haha"));
            case "<rule2>" -> result = rule.getRightSide().equals(RuleServiceTest.getFromString("cry"));
            default -> result = false;
        }
        return result;
    }

    @Test
    @Transactional
    public void jsgfWrite() throws IOException {

        String file1 = """
                #JSGF v 1.0;
                grammar first;
                                
                <rule1> = smile;
                //root;
                <rule2> = cry | sis;
                """;
        Grammar grammar = grammarService.processAddGrammar(new Grammar("first", null, null));
        ruleService.processAddRule(new Rule(null, "rule1", new HashSet<>(List.of(new RightSidePart("smile"))), false), grammar);
        ruleService.processAddRule(new Rule(null, "rule2", new HashSet<>(List.of(new RightSidePart("cry"), new RightSidePart("sis"))), true), grammar);
        Pair<FileHolder, Boolean> result = fileService.newFileFromGrammar(grammar.getId(), false);

        String output = Files.readString(Path.of("./grammars/" + result.getValue0().getFilePath()));

        System.out.print(output);
    }


    @Test
    @Transactional
    public void xmlWrite() throws IOException {

        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;
        Grammar grammar = grammarService.processAddGrammar(new Grammar("first", null, null));
        ruleService.processAddRule(new Rule(null, "rule1", new HashSet<>(List.of(new RightSidePart("smile"))), false), grammar);
        ruleService.processAddRule(new Rule(null, "rule2", new HashSet<>(List.of(new RightSidePart("cry"), new RightSidePart("sis"))), true), grammar);
        Pair<FileHolder, Boolean> result = fileService.newFileFromGrammar(grammar.getId(), true);

        String output = Files.readString(Path.of("./grammars/" + result.getValue0().getFilePath()));

        System.out.print(output);
    }


    @Test
    public void testGrammarEqualityFiles() throws IOException {
        grammarService.deleteAll();

        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;

        String file2 = """
                #JSGF V1.0;
                                
                grammar first;
                <rule1> = smile;
                <rule2> = cry|sis
                """;

        Grammar grammarFromXml = fileService.addGrammarFromFile(file1, true, null, false, false).getValue0();
        assert grammarFromXml.getFiles().size() == 0;
        Set<Rule> ruleSet = new HashSet<>();
        Rule rule1 = new Rule(null, "rule1", new HashSet<>(List.of(new RightSidePart("smile"))), false);
        Rule rule2 = new Rule(null, "rule2", new HashSet<>(List.of(new RightSidePart("cry|sis"))), false);
        ruleSet.add(rule1);
        ruleSet.add(rule2);
        Grammar grammarFromObj = new Grammar("first", ruleSet, null);

        grammarFromObj = grammarService.preProcessGrammar(grammarFromObj);
        grammarService.deleteAll();
        Grammar grammarFromJsgf = fileService.addGrammarFromFile(file2, false, null, false, false).getValue0();
        assert grammarFromJsgf.getFiles().size() == 0;

        assert grammarFromXml.equals(grammarFromObj);
        assert grammarFromJsgf.equals(grammarFromObj);
        assert grammarFromJsgf.equals(grammarFromXml);
    }

    @Test
    public void testFileGeneration() throws IOException {
        Set<Rule> ruleSet = new HashSet<>();
        Rule rule1 = new Rule(null, "rule1", new HashSet<>(List.of(new RightSidePart("smile"))), false);
        Rule rule2 = new Rule(null, "rule2", new HashSet<>(List.of(new RightSidePart("cry|sis"))), false);
        ruleSet.add(rule1);
        ruleSet.add(rule2);
        Grammar grammarFromObj = new Grammar("first", ruleSet, null);
        grammarFromObj = grammarService.processAddGrammar(grammarFromObj);
        Long id = grammarFromObj.getId();
        Pair<FileHolder, Boolean> result = fileService.newFileFromGrammar(id, false);
        assert result.getValue1();
        assert result.getValue0().getGrammar().equals(grammarFromObj);
        assert !fileService.newFileFromGrammar(id, false).getValue1();
        Pair<FileHolder, Boolean> result1 = fileService.newFileFromGrammar(id, true);
        assert result1.getValue1();
        assert result1.getValue0().getGrammar().equals(grammarFromObj);
        grammarService.changeUri(id, new Grammar("xd", null, null));
        assert fileService.newFileFromGrammar(id, false).getValue1();
        grammarService.deleteById(id);
        grammarFromObj = grammarService.processAddGrammar(grammarFromObj);
        fileService.newFileFromGrammar(grammarFromObj.getId(), false);
        grammarService.deleteAll();
    }


    @Test
    public void testFileSaving() throws IOException {
        grammarService.deleteAll();
        String fileXmlObj = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="firstObj"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;

        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;
        String fileXmlSameJsgf = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="first1"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;
        String file2 = """
                #JSGF V1.0;
                                
                grammar first1;
                <rule1> = smile;
                <rule2> = cry|sis
                """;
        Grammar grammarFromJsgf = fileService.addGrammarFromFile(file2, false, null, false, true).getValue0();
        Pair<Grammar, Boolean> resultFromXml = fileService.addGrammarFromFile(file1, true, null, false, true);
        Grammar grammarFromXml = resultFromXml.getValue0();
        assert resultFromXml.getValue1();
        assert grammarFromJsgf.getFiles().stream().toList().get(0).getGrammar().equals(grammarFromJsgf);
        assert grammarFromXml.getFiles().stream().toList().get(0).getGrammar().equals(grammarFromXml);
        String file2diff = """
                #JSGF V1.0;
                                
                grammar first12;
                <rule1> = smile;
                <rule2> = crfy|sis
                """;
        Set<Rule> ruleSet = new HashSet<>();
        Rule rule1 = new Rule(null, "rule1", new HashSet<>(List.of(new RightSidePart("smile"))), false);
        Rule rule2 = new Rule(null, "rule2", new HashSet<>(List.of(new RightSidePart("cry|sis"))), false);
        ruleSet.add(rule1);
        ruleSet.add(rule2);
        Grammar grammarFromObj = new Grammar("firstObj", ruleSet, null);
        grammarFromObj = grammarService.processAddGrammar(grammarFromObj);
        Long id = grammarFromObj.getId();
        assert fileService.addGrammarFromFile(file2diff, false, grammarFromJsgf.getId(), false, true) == null;
        Pair<Grammar, Boolean> resXml2 = fileService.addGrammarFromFile(fileXmlSameJsgf, true, grammarFromJsgf.getId(), false, true);
        Grammar grammarFromXml2 = resXml2.getValue0();
        assert !resXml2.getValue1();
        assert grammarFromXml2.getFiles().stream().toList().size() == 2;
        Pair<Grammar, Boolean> resFromObj2 = fileService.addGrammarFromFile(fileXmlObj, true, id, false, true);
        Grammar grammarFromObj2 = resFromObj2.getValue0();
        assert !resFromObj2.getValue1();
        assert grammarFromObj2.getFiles().stream().toList().size() == 1;
        assert grammarFromObj2.getFiles().stream().toList().get(0).getGrammar().equals(grammarFromObj2);
        Pair<Grammar, Boolean> resFromJsgf2 = fileService.addGrammarFromFile(file2diff, false, grammarFromJsgf.getId(), true, true);
        Grammar grammarFromJsgf2 = resFromJsgf2.getValue0();
        assert resFromJsgf2.getValue1();
        assert grammarFromJsgf2.getFiles().stream().toList().size() == 1;
        assert grammarFromJsgf2.getFiles().stream().toList().get(0).getGrammar().equals(grammarFromJsgf2);
        grammarService.deleteAll();
    }


    @Test
    public void testFileDelete() throws IOException {
        grammarService.deleteAll();
        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;
        Grammar grammarFromJsgf = fileService.addGrammarFromFile(file1, true, null, false, true).getValue0();
        fileService.deleteFilesFromGrammar(grammarFromJsgf.getId());
        assert fileService.getAllFiles().size() == 0;
        grammarFromJsgf = fileService.addGrammarFromFile(file1, true, null, false, true).getValue0();
        Long id = grammarFromJsgf.getFiles().stream().toList().get(0).getId();
        fileService.deleteFileById(id);
        assert fileService.getAllFiles().size() == 0;
    }
}
